### Name: nnls
### Title: The Lawson-Hanson NNLS algorithm for non-negative least squares
### Aliases: nnls
### Keywords: optimize

### ** Examples

## simulate a matrix A 
t <- seq(0, 2, by = .04)
k <- c(.5, .6, 1)
A <- matrix(nrow = 51, ncol = 3)
Acolfunc <- function(k, t) exp(-k*t)
for(i in 1:3) A[,i] <- Acolfunc(k[i],t)

## simulate a data vector b based on A 
E <- matrix(nrow = 51, ncol = 3)
wavenum <- seq(18000,28000, by=200)
location <- c(25000, 22000, 20000) 
delta <- c(3000,3000,3000)
Ecolfunc <- function(wavenum, location, delta)
  exp( - log(2) * (2 * (wavenum - location)/delta)^2)
for(i in 1:3) E[,i] <- Ecolfunc(wavenum, location[i], delta[i])
set.seed(3312)
matdat <- A %*% t(E) + .01 * rnorm(nrow(A) * nrow(E))
## this is the data vector b
b <- matdat[,20]

## these are the x values to recover
realx <- E[20,] 

## by least squares criteria, there are negative values in x 
qr.coef(qr(A),b)  

## -------- CALL TO NNLS ------
## solve A x = b subject to x >= 0 
(x <- nnls(A = A, b = b))
## ----------------------------

## Not run: 
##D 
##D ## can solve the same problem with L-BFGS-B algorithm
##D ## but need starting values for x 
##D startval <- rep(0, ncol(A))
##D fn1 <- function(par1) sum( ( b - A %*% par1)^2)
##D (x_lbfgsb <- optim(startval, fn = fn1, lower = rep(0,3),
##D                   method="L-BFGS-B")$par)
##D 
##D ## RMS deviation (xreal - x_lbfgsb) > RMS deviation (xreal - x)
##D sqrt(sum((realx - x)^2)) < sqrt(sum((realx - x_lbfgsb)^2))
##D 
##D ## and L-BFGS-B is much slower 
##D nnlsrep <- function() for(i in 1:100) nnls(A = A, b = b)
##D lbfgsbrep <- function() for(i in 1:100) optim(startval, fn = fn1,
##D                                   lower = rep(0, 3), method="L-BFGS-B")
##D (nnls_time <- system.time(nnlsrep()))
##D (lbfgsb_time <- system.time(lbfgsbrep()))
##D 
##D ## can also solve the same problem by reformulating it as a
##D ## quadratic program (this requires the quadprog package; if you
##D ## have quadprog installed, uncomment lines below starting with
##D ## only 1 "#" )
##D 
##D # library(quadprog)
##D # dvec <- crossprod(A,b) 
##D # bvec <- rep(0, ncol(A))
##D # Dmat <- crossprod(A,A)
##D # Amat <- diag(ncol(A))
##D # (x_qp <- solve.QP(dvec = dvec, bvec = bvec, Dmat=Dmat,
##D          Amat=Amat)$solution)
##D 
##D ## RMS deviation (xreal - x_qp) > RMS deviation (xreal - x)
##D # sqrt(sum((realx - x)^2)) < sqrt(sum((realx - x_qp)^2)) 
##D 
##D # qprep <- function() for(i in 1:100) solve.QP(dvec = dvec, bvec = bvec,
##D #                                     Dmat = Dmat, Amat = Amat)
##D ## and quadprog is also slower, even not taking into account time
##D ## for problem reformulation
##D # (qp_time <- system.time(qprep()))
## End(Not run)




